@@ -155,8 +155,8 @@ rsvg_start_style (RsvgHandle *ctx, RsvgPropertyBag *atts)
 
 
 static void
-rsvg_filter_handler_start (RsvgHandle *ctx, const xmlChar *name,
-						   RsvgPropertyBag *atts)
+rsvg_standard_element_start (RsvgHandle *ctx, const xmlChar *name,
+							 RsvgPropertyBag *atts)
 {
 
 	/*replace this stuff with a hash for fast reading!*/
@@ -265,9 +265,15 @@ rsvg_filter_handler_start (RsvgHandle *ctx, const xmlChar *name,
 		newnode = rsvg_new_image();
 	else if (!strcmp ((char *)name, "subImage"))
 		newnode = rsvg_new_group();
-
+	else if (!strcmp ((char *)name, "text"))
+		newnode = rsvg_new_text();
+	else if (!strcmp ((char *)name, "tspan"))
+		newnode = rsvg_new_tspan();
+	else if (!strcmp ((char *)name, "tref"))
+		newnode = rsvg_new_tref();
 	if (newnode)
 		{
+			newnode->type = g_string_new((char *)name);
 			rsvg_node_set_atts(newnode, ctx, atts);
 			rsvg_defs_register_memory(ctx->defs, newnode);
 			if (ctx->currentnode) {
@@ -528,13 +534,6 @@ rsvg_start_element (void *data, const xmlChar *name,
 	RsvgHandle *ctx = (RsvgHandle *)data;
 
 	RsvgPropertyBag * bag;
-  
-	RsvgDimensionData * newdimension;
-	newdimension = g_new(RsvgDimensionData, 1);
-	newdimension->width = ctx->width;
-	newdimension->height = ctx->height;
-	newdimension->em = rsvg_state_current_font_size(ctx);
-	ctx->dimensions = g_slist_prepend(ctx->dimensions, newdimension);
 
 	bag = rsvg_property_bag_new(atts);
 
@@ -551,9 +550,7 @@ rsvg_start_element (void *data, const xmlChar *name,
 				if (*tempname == ':')
 					name = tempname + 1;
 			
-			if (!strcmp ((char *)name, "text"))
-				rsvg_start_text (ctx, bag);
-			else if (!strcmp ((char *)name, "style"))
+			if (!strcmp ((char *)name, "style"))
 				rsvg_start_style (ctx, bag);
 			else if (!strcmp ((char *)name, "title"))
 				rsvg_start_title (ctx, bag);
@@ -561,7 +558,7 @@ rsvg_start_element (void *data, const xmlChar *name,
 				rsvg_start_desc (ctx, bag);
 			else if (!strcmp ((char *)name, "metadata"))
 				rsvg_start_metadata (ctx, bag);
-			rsvg_filter_handler_start (ctx, name, bag);
+			rsvg_standard_element_start (ctx, name, bag);
     }
 
 	rsvg_property_bag_free(bag);
@@ -571,13 +568,6 @@ static void
 rsvg_end_element (void *data, const xmlChar *name)
 {
 	RsvgHandle *ctx = (RsvgHandle *)data;
-	
-	GSList * link = g_slist_nth(ctx->dimensions, 0);
-	RsvgDimensionData * dead_dimension = (RsvgDimensionData *)link->data;
-	ctx->width = dead_dimension->width;
-	ctx->height = dead_dimension->height;
-	g_free (dead_dimension);
-	ctx->dimensions = g_slist_delete_link(ctx->dimensions, link);
 
 	if (ctx->handler_nest > 0 && ctx->handler != NULL)
 		{
@@ -597,47 +587,18 @@ rsvg_end_element (void *data, const xmlChar *name)
 					ctx->handler = NULL;
 				}
 
-			if (!strcmp ((char *)name, "image") ||
-				!strcmp ((char *)name, "use") ||
-				!strcmp ((char *)name, "switch") ||
-				!strcmp ((char *)name, "marker") ||
-				!strcmp ((char *)name, "clipPath") ||
-				!strcmp ((char *)name, "mask") ||
-				!strcmp ((char *)name, "defs") ||
-				!strcmp ((char *)name, "filter") ||
-				!strcmp ((char *)name, "symbol") ||
-				!strcmp ((char *)name, "svg") ||
-				!strcmp ((char *)name, "a") ||
-				!strcmp ((char *)name, "line") ||
-				!strcmp ((char *)name, "rect") ||
-				!strcmp ((char *)name, "circle") ||
-				!strcmp ((char *)name, "ellipse") ||
-				!strcmp ((char *)name, "polyline") ||
-				!strcmp ((char *)name, "polygon") ||
-				!strcmp ((char *)name, "path") ||
-				!strcmp ((char *)name, "g") ||
-				!strcmp ((char *)name, "pattern") ||
-				!strcmp ((char *)name, "linearGradient") ||
-				!strcmp ((char *)name, "radialGradient") ||
-				!strcmp ((char *)name, "conicalGradient") ||
-				!strcmp ((char *)name, "stop") ||
-				!strncmp ((char *)name, "fe", 2))
-				{
-					/*when type enums are working right we should test if the end is the same as the current node*/
-					rsvg_pop_def_group(ctx);
-				}
+			if (!strcmp ((char *)name, ctx->currentnode->type->str))
+				rsvg_pop_def_group(ctx);
 			
 		}
 }
 
-#if 0
 static void _rsvg_node_chars_free(RsvgNode * node)
 {
 	RsvgNodeChars * self = (RsvgNodeChars *)node;
 	g_string_free(self->contents, TRUE);
 	_rsvg_node_free(node);
 }
-#endif
 
 static void
 rsvg_characters (void *data, const xmlChar *ch, int len)
@@ -650,7 +611,6 @@ rsvg_characters (void *data, const xmlChar *ch, int len)
 			return;
 		}
 
-#if 0
 	char * utf8 = NULL;
 	RsvgNodeChars * self;
 	GString * string;
@@ -670,13 +630,13 @@ rsvg_characters (void *data, const xmlChar *ch, int len)
 	_rsvg_node_init(&self->super);
 	self->contents = string;
 
-	self->super.type = RSVG_NODE_CHARS;
+	self->super.type = g_string_new("RSVG_NODE_CHARS");
 	self->super.free = _rsvg_node_chars_free;
+	self->super.state->cond_true = FALSE;
 
 	rsvg_defs_register_memory(ctx->defs, (RsvgNode *)self);
 	if (ctx->currentnode)
 		rsvg_node_group_pack(ctx->currentnode, (RsvgNode *)self);
-#endif
 }
 
 #if LIBXML_VERSION >= 20621
@@ -1033,7 +993,6 @@ rsvg_handle_new (void)
 	handle->currentnode = NULL;
 	handle->treebase = NULL;
 
-	handle->dimensions = NULL;
 	handle->finished = 0;
 	handle->first_write = TRUE;
 
@@ -1171,48 +1130,32 @@ void
 rsvg_handle_get_dimensions(RsvgHandle * handle, RsvgDimensionData * output)
 {
 	RsvgNodeSvg * sself;
+	RsvgBbox bbox;
 
 	sself = (RsvgNodeSvg *)handle->treebase;	
 	if(!sself) {
 		memset(output, 0, sizeof(RsvgDimensionData));
 		return;
 	}
-
-	if (sself->hasw && sself->hash)
+	
+	bbox.x = bbox.y = 0;
+	bbox.w = bbox.h = 1;
+	
+	if (sself->w.factor == 'p' || sself->h.factor == 'p')
 		{
-			RsvgBbox bbox;
-			
-			bbox.x = bbox.y = 0;
-			bbox.w = bbox.h = 1;
-
-			if (sself->w.factor == 'p' || sself->h.factor == 'p')
+			if (sself->vbox.active && sself->vbox.w > 0. && sself->vbox.h > 0.)
 				{
-					if (sself->has_vbox && sself->vbw > 0. && sself->vbh > 0.)
-						{
-							bbox.w = sself->vbw;
-							bbox.h = sself->vbh;
-						}
-					else
-						bbox = _rsvg_find_bbox(handle);
+					bbox.w = sself->vbox.w;
+					bbox.h = sself->vbox.h;
 				}
-			output->width  = _rsvg_css_hand_normalize_length_struct(&sself->w, handle->dpi_x, 
+			else
+				bbox = _rsvg_find_bbox(handle);
+		}
+	output->width  = _rsvg_css_hand_normalize_length(&sself->w, handle->dpi_x, 
 																	bbox.w + bbox.x * 2, 12);
-			output->height = _rsvg_css_hand_normalize_length_struct(&sself->h, handle->dpi_y, 
-																	bbox.h + bbox.y * 2, 12);
-		}
-	else if (sself->has_vbox && sself->vbw > 0. && sself->vbh > 0.)
-		{
-			output->width  = (int)floor (sself->vbw);
-			output->height = (int)floor (sself->vbh);
-		}
-	else
-		{
-			RsvgBbox bbox;
-			bbox = _rsvg_find_bbox(handle);
-			output->width  = bbox.w + bbox.x * 2;
-			output->height = bbox.h + bbox.y * 2;
-		}
-
+	output->height = _rsvg_css_hand_normalize_length(&sself->h, handle->dpi_y, 
+													 bbox.h + bbox.y * 2, 12);
+	
 	output->em = output->width;
 	output->ex = output->height;
 
@@ -1223,6 +1166,22 @@ rsvg_handle_get_dimensions(RsvgHandle * handle, RsvgDimensionData * output)
 
 /** 
  * rsvg_set_default_dpi
+ * @dpi: Dots Per Inch (aka Pixels Per Inch)
+ *
+ * Sets the DPI for the all future outgoing pixbufs. Common values are
+ * 75, 90, and 300 DPI. Passing a number <= 0 to #dpi will 
+ * reset the DPI to whatever the default value happens to be.
+ *
+ * Since: 2.8
+ */
+void
+rsvg_set_default_dpi (double dpi)
+{
+	rsvg_set_default_dpi_x_y (dpi, dpi);
+}
+
+/** 
+ * rsvg_set_default_dpi_x_y
  * @dpi_x: Dots Per Inch (aka Pixels Per Inch)
  * @dpi_y: Dots Per Inch (aka Pixels Per Inch)
  *
@@ -1233,7 +1192,7 @@ rsvg_handle_get_dimensions(RsvgHandle * handle, RsvgDimensionData * output)
  * Since: 2.8
  */
 void
-rsvg_set_default_dpi (double dpi_x, double dpi_y)
+rsvg_set_default_dpi_x_y (double dpi_x, double dpi_y)
 {
 	if (dpi_x <= 0.)
 		internal_dpi_x = RSVG_DEFAULT_DPI_X;
@@ -1246,6 +1205,23 @@ rsvg_set_default_dpi (double dpi_x, double dpi_y)
 		internal_dpi_y = dpi_y;
 }
 
+/**
+ * rsvg_handle_set_dpi
+ * @handle: An #RsvgHandle
+ * @dpi: Dots Per Inch (aka Pixels Per Inch)
+ *
+ * Sets the DPI for the outgoing pixbuf. Common values are
+ * 75, 90, and 300 DPI. Passing a number <= 0 to #dpi will 
+ * reset the DPI to whatever the default value happens to be.
+ *
+ * Since: 2.8
+ */
+void
+rsvg_handle_set_dpi (RsvgHandle * handle, double dpi)
+{
+	rsvg_handle_set_dpi_x_y (handle, dpi, dpi);
+}
+
 /**
  * rsvg_handle_set_dpi
  * @handle: An #RsvgHandle
@@ -1259,7 +1235,7 @@ rsvg_set_default_dpi (double dpi_x, double dpi_y)
  * Since: 2.8
  */
 void
-rsvg_handle_set_dpi (RsvgHandle * handle, double dpi_x, double dpi_y)
+rsvg_handle_set_dpi_x_y (RsvgHandle * handle, double dpi_x, double dpi_y)
 {
 	g_return_if_fail (handle != NULL);
 	
