@@ -95,7 +95,6 @@ rsvg_new_path (void)
 	path = g_new (RsvgNodePath, 1);
 	_rsvg_node_init(&path->super);
 	path->d = NULL;
-	path->super.type = RSVG_NODE_PATH;
 	path->super.free = rsvg_node_path_free;
 	path->super.draw = rsvg_node_path_draw;
 	path->super.set_atts = rsvg_node_path_set_atts;
@@ -147,19 +146,30 @@ _rsvg_node_poly_draw(RsvgNode * self, RsvgDrawingCtx *ctx,
 	RsvgNodePoly * poly = (RsvgNodePoly *)self;
 	gsize i;
 	GString * d;
+	char buf [G_ASCII_DTOSTR_BUF_SIZE];
 
 	/* represent as a "moveto, lineto*, close" path */  
 	if (poly->pointlist_len < 2)
 		return;
 
 	d = g_string_new (NULL);
-	g_string_append_printf (d, "M %f %f ", poly->pointlist[0], poly->pointlist[1] );
-	
-	for (i = 2; i < poly->pointlist_len; i += 2)
-		g_string_append_printf (d, "L %f %f ", poly->pointlist[i], poly->pointlist[i+1]);
-	
+
+	/* 	"M %f %f " */
+	g_string_append (d, " M ");	
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), poly->pointlist[0]));
+	g_string_append_c (d, ' ');
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), poly->pointlist[1]));
+
+	/* "L %f %f " */
+	for (i = 2; i < poly->pointlist_len; i += 2) {
+		g_string_append (d, " L ");	
+		g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), poly->pointlist[i]));
+		g_string_append_c (d, ' ');
+		g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), poly->pointlist[i+1]));
+	}
+
 	if (!poly->is_polyline)
-		g_string_append (d, "Z");
+		g_string_append (d, " Z");
 	
 	rsvg_state_reinherit_top(ctx, self->state, dominate);
 	rsvg_render_path (ctx, d->str);
@@ -185,7 +195,6 @@ rsvg_new_any_poly(gboolean is_polyline)
 	RsvgNodePoly *poly;
 	poly = g_new (RsvgNodePoly, 1);
 	_rsvg_node_init(&poly->super);
-	poly->super.type = RSVG_NODE_PATH;
 	poly->super.free = _rsvg_node_poly_free;
 	poly->super.draw = _rsvg_node_poly_draw;
 	poly->super.set_atts = _rsvg_node_poly_set_atts;
@@ -211,7 +220,7 @@ rsvg_new_polyline (void)
 struct _RsvgNodeLine
 {
 	RsvgNode super;
-	double x1, x2, y1, y2;
+	RsvgLength x1, x2, y1, y2;
 };
 
 typedef struct _RsvgNodeLine RsvgNodeLine;
@@ -220,22 +229,18 @@ static void
 _rsvg_node_line_set_atts (RsvgNode * self, RsvgHandle *ctx, RsvgPropertyBag *atts)
 {
 	const char * klazz = NULL, *id = NULL, *value;
-	double font_size;
 	RsvgNodeLine * line = (RsvgNodeLine *) self;
 
-	font_size = rsvg_state_current_font_size (ctx);
-	
-
 	if (rsvg_property_bag_size (atts))
 		{
 			if ((value = rsvg_property_bag_lookup (atts, "x1")))
-				line->x1 = rsvg_css_parse_normalized_length (value, ctx->dpi_x, (gdouble)ctx->width, font_size);
+				line->x1 = _rsvg_css_parse_length (value);
 			if ((value = rsvg_property_bag_lookup (atts, "y1")))
-				line->y1 = rsvg_css_parse_normalized_length (value, ctx->dpi_y, (gdouble)ctx->height, font_size);
+				line->y1 = _rsvg_css_parse_length (value);
 			if ((value = rsvg_property_bag_lookup (atts, "x2")))
-				line->x2 = rsvg_css_parse_normalized_length (value, ctx->dpi_x, (gdouble)ctx->width, font_size);
+				line->x2 = _rsvg_css_parse_length (value);
 			if ((value = rsvg_property_bag_lookup (atts, "y2")))
-				line->y2 = rsvg_css_parse_normalized_length (value, ctx->dpi_y, (gdouble)ctx->height, font_size);
+				line->y2 = _rsvg_css_parse_length (value);
 			if ((value = rsvg_property_bag_lookup (atts, "class")))
 				klazz = value;
 			if ((value = rsvg_property_bag_lookup (atts, "id")))
@@ -260,13 +265,17 @@ _rsvg_node_line_draw(RsvgNode * overself, RsvgDrawingCtx *ctx,
 	/* ("M %f %f L %f %f", x1, y1, x2, y2) */
 	d = g_string_new ("M ");   
 
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), self->x1));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), 
+										_rsvg_css_normalize_length(&self->x1,ctx,'h')));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), self->y1));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), 
+										_rsvg_css_normalize_length(&self->y1,ctx,'v')));
 	g_string_append (d, " L ");	
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), self->x2));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), 
+										_rsvg_css_normalize_length(&self->x2,ctx,'h')));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), self->y2));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), 
+										_rsvg_css_normalize_length(&self->y2,ctx,'v')));
 
 	rsvg_state_reinherit_top(ctx, overself->state, dominate);
 	rsvg_render_path (ctx, d->str);
@@ -280,17 +289,16 @@ rsvg_new_line (void)
 	RsvgNodeLine *line;
 	line = g_new (RsvgNodeLine, 1);
 	_rsvg_node_init(&line->super);
-	line->super.type = RSVG_NODE_PATH;
 	line->super.draw = _rsvg_node_line_draw;
 	line->super.set_atts = _rsvg_node_line_set_atts;
-	line->x1 = line->x2 = line->y1 = line->y2 = 0;
+	line->x1 = line->x2 = line->y1 = line->y2 = _rsvg_css_parse_length("0");
 	return &line->super;
 }
 
 struct _RsvgNodeRect
 {
 	RsvgNode super;
-	double x, y, w, h, rx, ry;
+	RsvgLength x, y, w, h, rx, ry;
 	gboolean got_rx, got_ry;
 };
 
@@ -300,27 +308,24 @@ static void
 _rsvg_node_rect_set_atts (RsvgNode * self, RsvgHandle *ctx, RsvgPropertyBag *atts)
 {
 	const char * klazz = NULL, * id = NULL, *value;
-	double font_size;
 	RsvgNodeRect * rect = (RsvgNodeRect *)self;
 
-	font_size = rsvg_state_current_font_size (ctx);
-
 	if (rsvg_property_bag_size (atts))
 		{
 			if ((value = rsvg_property_bag_lookup (atts, "x")))
-				rect->x = rsvg_css_parse_normalized_length (value, ctx->dpi_x, (gdouble)ctx->width, font_size);
+				rect->x = _rsvg_css_parse_length (value);
 			if ((value = rsvg_property_bag_lookup (atts, "y")))
-				rect->y = rsvg_css_parse_normalized_length (value, ctx->dpi_y, (gdouble)ctx->height, font_size) + 0.01;
+				rect->y = _rsvg_css_parse_length (value);
 			if ((value = rsvg_property_bag_lookup (atts, "width")))
-				rect->w = rsvg_css_parse_normalized_length (value, ctx->dpi_x, (gdouble)ctx->width, font_size);
+				rect->w = _rsvg_css_parse_length (value);
 			if ((value = rsvg_property_bag_lookup (atts, "height")))
-				rect->h = rsvg_css_parse_normalized_length (value, ctx->dpi_y, (gdouble)ctx->height, font_size);
+				rect->h = _rsvg_css_parse_length (value);
 			if ((value = rsvg_property_bag_lookup (atts, "rx"))) {
-				rect->rx = rsvg_css_parse_normalized_length (value, ctx->dpi_x, (gdouble)ctx->width, font_size);
+				rect->rx = _rsvg_css_parse_length (value);
 				rect->got_rx = TRUE;	
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "ry"))) {
-				rect->ry = rsvg_css_parse_normalized_length (value, ctx->dpi_y, (gdouble)ctx->height, font_size);
+				rect->ry = _rsvg_css_parse_length (value);
 				rect->got_ry = TRUE;
 			}
 			if ((value = rsvg_property_bag_lookup (atts, "class")))
@@ -339,33 +344,40 @@ static void
 _rsvg_node_rect_draw(RsvgNode * self, RsvgDrawingCtx *ctx, 
 					 int dominate)
 {
-	double rx, ry;	
+	double x, y, w, h, rx, ry;	
 	GString * d = NULL;
 	RsvgNodeRect * rect = (RsvgNodeRect *)self;
 	char buf [G_ASCII_DTOSTR_BUF_SIZE];
 
-	if (rect->got_rx)
-		rx = rect->rx;		
-	else
-		rx = rect->ry;
-	if (rect->got_ry)
-		ry = rect->ry;		
-	else
-		ry = rect->rx;
+	x = _rsvg_css_normalize_length(&rect->x, ctx, 'h');
+	y = _rsvg_css_normalize_length(&rect->y, ctx, 'v');
+	w = _rsvg_css_normalize_length(&rect->w, ctx, 'h');
+	h = _rsvg_css_normalize_length(&rect->h, ctx, 'v');
+	rx = _rsvg_css_normalize_length(&rect->rx, ctx, 'h');
+	ry = _rsvg_css_normalize_length(&rect->ry, ctx, 'v');
 
-	if (rx > fabs(rect->w / 2.))
-		rx = fabs(rect->w / 2.);
-	if (ry > fabs(rect->h / 2.))
-		ry = fabs(rect->h / 2.);
+	if (rect->got_rx)
+		rx = rx;		
+	else
+		rx = ry;
+	if (rect->got_ry)
+		ry = ry;		
+	else
+		ry = rx;
+
+	if (rx > fabs(w / 2.))
+		rx = fabs(w / 2.);
+	if (ry > fabs(h / 2.))
+		ry = fabs(h / 2.);
 
 	/* emulate a rect using a path */
 	d = g_string_new ("M ");
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), rect->x + rx));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), x + rx));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), rect->y));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), y));
 
 	g_string_append (d, " H ");
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), rect->x + rect->w - rx));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), x + w - rx));
 
 	g_string_append (d, " A");
 	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), rx));
@@ -378,12 +390,12 @@ _rsvg_node_rect_draw(RsvgNode * self, RsvgDrawingCtx *ctx,
 	g_string_append_c (d, ' ');
 	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), 1.));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), rect->x+rect->w));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), x+w));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), rect->y+ry));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), y+ry));
 
 	g_string_append (d, " V ");
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), rect->y+rect->h-ry));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), y+h-ry));
 
 	g_string_append (d, " A");
 	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), rx));
@@ -396,12 +408,12 @@ _rsvg_node_rect_draw(RsvgNode * self, RsvgDrawingCtx *ctx,
 	g_string_append_c (d, ' ');
 	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), 1.));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), rect->x + rect->w - rx));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), x + w - rx));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), rect->y + rect->h));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), y + h));
 
 	g_string_append (d, " H ");
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), rect->x + rx));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), x + rx));
 
 	g_string_append (d, " A");
 	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), rx));
@@ -414,12 +426,12 @@ _rsvg_node_rect_draw(RsvgNode * self, RsvgDrawingCtx *ctx,
 	g_string_append_c (d, ' ');	
 	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), 1.));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), rect->x));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), x));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), rect->y +rect-> h - ry));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), y + h - ry));
 
 	g_string_append (d, " V ");
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), rect->y+ry));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), y+ry));
 
 	g_string_append (d, " A");
 	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), rx));
@@ -432,9 +444,9 @@ _rsvg_node_rect_draw(RsvgNode * self, RsvgDrawingCtx *ctx,
 	g_string_append_c (d, ' ');
 	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), 1.));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), rect->x+rx));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), x+rx));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), rect->y));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), y));
 
 	g_string_append (d, " Z");
 
@@ -449,10 +461,10 @@ rsvg_new_rect (void)
 	RsvgNodeRect *rect;
 	rect = g_new (RsvgNodeRect, 1);
 	_rsvg_node_init(&rect->super);
-	rect->super.type = RSVG_NODE_PATH;
 	rect->super.draw = _rsvg_node_rect_draw;
 	rect->super.set_atts = _rsvg_node_rect_set_atts;
-	rect->x = rect->y = rect->w = rect->h = rect->rx = rect->ry = 0;
+	rect->x = rect->y = rect->w = rect->h = rect->rx = rect->ry = 
+		_rsvg_css_parse_length("0");
 	rect->got_rx = rect->got_ry = FALSE;
 	return &rect->super;
 }
@@ -460,7 +472,7 @@ rsvg_new_rect (void)
 struct _RsvgNodeCircle
 {
 	RsvgNode super;
-	double cx, cy, r;
+	RsvgLength cx, cy, r;
 };
 
 typedef struct _RsvgNodeCircle RsvgNodeCircle;
@@ -469,21 +481,16 @@ static void
 _rsvg_node_circle_set_atts (RsvgNode * self, RsvgHandle *ctx, RsvgPropertyBag *atts)
 {
 	const char * klazz = NULL, * id = NULL, *value;
-	double font_size;
 	RsvgNodeCircle * circle = (RsvgNodeCircle *)self;
 
-	font_size = rsvg_state_current_font_size (ctx);
-
 	if (rsvg_property_bag_size (atts))
 		{
 			if ((value = rsvg_property_bag_lookup (atts, "cx")))
-				circle->cx = rsvg_css_parse_normalized_length (value, ctx->dpi_x, (gdouble)ctx->width, font_size);
+				circle->cx = _rsvg_css_parse_length (value);
 			if ((value = rsvg_property_bag_lookup (atts, "cy")))
-				circle->cy = rsvg_css_parse_normalized_length (value, ctx->dpi_y, (gdouble)ctx->height, font_size);
+				circle->cy = _rsvg_css_parse_length (value);
 			if ((value = rsvg_property_bag_lookup (atts, "r")))
-				circle->r = rsvg_css_parse_normalized_length (value, rsvg_dpi_percentage (ctx), 
-															  rsvg_viewport_percentage((gdouble)ctx->width, (gdouble)ctx->height), 
-															  font_size);
+				circle->r = _rsvg_css_parse_length (value);
 			if ((value = rsvg_property_bag_lookup (atts, "class")))
 				klazz = value;
 			if ((value = rsvg_property_bag_lookup (atts, "id")))
@@ -503,68 +510,73 @@ _rsvg_node_circle_draw(RsvgNode * self, RsvgDrawingCtx *ctx,
 	GString * d = NULL;
 	RsvgNodeCircle * circle = (RsvgNodeCircle *)self;
 	char buf [G_ASCII_DTOSTR_BUF_SIZE];
+	double cx, cy, r;
 
-	if (circle->r <= 0)
+	cx = _rsvg_css_normalize_length(&circle->cx, ctx, 'h');
+	cy = _rsvg_css_normalize_length(&circle->cy, ctx, 'v');
+	r = _rsvg_css_normalize_length(&circle->r, ctx, 'o');
+
+	if (r <= 0)
 		return;
 	
 	/* approximate a circle using 4 bezier curves */
 
 	d = g_string_new ("M ");
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), circle->cx+circle->r));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cx+r));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), circle->cy));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cy));
 
 	g_string_append (d, " C ");
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), circle->cx+circle->r));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cx+r));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), circle->cy + circle->r * RSVG_ARC_MAGIC));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cy + r * RSVG_ARC_MAGIC));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), circle->cx + circle->r * RSVG_ARC_MAGIC));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cx + r * RSVG_ARC_MAGIC));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), circle->cy + circle->r));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cy + r));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), circle->cx));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cx));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), circle->cy + circle->r));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cy + r));
 
 	g_string_append (d, " C ");
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), circle->cx - circle->r * RSVG_ARC_MAGIC));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cx - r * RSVG_ARC_MAGIC));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), circle->cy + circle->r));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cy + r));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), circle->cx - circle->r));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cx - r));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), circle->cy + circle->r * RSVG_ARC_MAGIC));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cy + r * RSVG_ARC_MAGIC));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), circle->cx - circle->r));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cx - r));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), circle->cy));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cy));
 
 	g_string_append (d, " C ");
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), circle-> cx - circle->r));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf),  cx - r));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), circle->cy - circle->r * RSVG_ARC_MAGIC));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cy - r * RSVG_ARC_MAGIC));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), circle->cx - circle->r * RSVG_ARC_MAGIC));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cx - r * RSVG_ARC_MAGIC));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), circle->cy - circle->r));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cy - r));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), circle->cx));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cx));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), circle->cy - circle->r));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cy - r));
 
 	g_string_append (d, " C ");
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), circle->cx + circle->r * RSVG_ARC_MAGIC));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cx + r * RSVG_ARC_MAGIC));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), circle->cy - circle->r));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cy - r));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), circle->cx + circle->r));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cx + r));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), circle->cy - circle->r * RSVG_ARC_MAGIC));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cy - r * RSVG_ARC_MAGIC));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), circle->cx + circle->r));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cx + r));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), circle->cy));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cy));
 
 	g_string_append (d, " Z");
 
@@ -580,17 +592,16 @@ rsvg_new_circle (void)
 	RsvgNodeCircle *circle;
 	circle = g_new (RsvgNodeCircle, 1);
 	_rsvg_node_init(&circle->super);
-	circle->super.type = RSVG_NODE_PATH;
 	circle->super.draw = _rsvg_node_circle_draw;
 	circle->super.set_atts = _rsvg_node_circle_set_atts;
-	circle->cx = circle->cy = circle->r = 0;
+	circle->cx = circle->cy = circle->r = _rsvg_css_parse_length("0");
 	return &circle->super;
 }
 
 struct _RsvgNodeEllipse
 {
 	RsvgNode super;
-	double cx, cy, rx, ry;
+	RsvgLength cx, cy, rx, ry;
 };
 
 typedef struct _RsvgNodeEllipse RsvgNodeEllipse;
@@ -599,21 +610,18 @@ static void
 _rsvg_node_ellipse_set_atts (RsvgNode * self, RsvgHandle *ctx, RsvgPropertyBag *atts)
 {
 	const char * klazz = NULL, * id = NULL, *value;
-	double font_size;
 	RsvgNodeEllipse * ellipse = (RsvgNodeEllipse *)self;
 
-	font_size = rsvg_state_current_font_size (ctx);
-
 	if (rsvg_property_bag_size (atts))
 		{
 			if ((value = rsvg_property_bag_lookup (atts, "cx")))
-				ellipse->cx = rsvg_css_parse_normalized_length (value, ctx->dpi_x, (gdouble)ctx->width, font_size);
+				ellipse->cx = _rsvg_css_parse_length (value);
 			if ((value = rsvg_property_bag_lookup (atts, "cy")))
-				ellipse->cy = rsvg_css_parse_normalized_length (value, ctx->dpi_y, (gdouble)ctx->height, font_size);
+				ellipse->cy = _rsvg_css_parse_length (value);
 			if ((value = rsvg_property_bag_lookup (atts, "rx")))
-				ellipse->rx = rsvg_css_parse_normalized_length (value, ctx->dpi_x, (gdouble)ctx->width, font_size);
+				ellipse->rx = _rsvg_css_parse_length (value);
 			if ((value = rsvg_property_bag_lookup (atts, "ry")))
-				ellipse->ry = rsvg_css_parse_normalized_length (value, ctx->dpi_y, (gdouble)ctx->height, font_size);
+				ellipse->ry = _rsvg_css_parse_length (value);
 			if ((value = rsvg_property_bag_lookup (atts, "class")))
 				klazz = value;
 			if ((value = rsvg_property_bag_lookup (atts, "id")))
@@ -633,67 +641,73 @@ _rsvg_node_ellipse_draw(RsvgNode * self, RsvgDrawingCtx *ctx,
 	RsvgNodeEllipse * ellipse = (RsvgNodeEllipse *)self;
 	GString * d = NULL;
 	char buf [G_ASCII_DTOSTR_BUF_SIZE];
+	double cx, cy, rx, ry;
+
+	cx = _rsvg_css_normalize_length(&ellipse->cx, ctx, 'h');
+	cy = _rsvg_css_normalize_length(&ellipse->cy, ctx, 'v');
+	rx = _rsvg_css_normalize_length(&ellipse->rx, ctx, 'h');
+	ry = _rsvg_css_normalize_length(&ellipse->ry, ctx, 'v');
 	
-	if (ellipse->rx <= 0 || ellipse->ry <= 0)
+	if (rx <= 0 || ry <= 0)
 		return;
 	/* approximate an ellipse using 4 bezier curves */
 
 	d = g_string_new ("M ");
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), ellipse->cx + ellipse->rx));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cx + rx));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), ellipse->cy));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cy));
 
 	g_string_append (d, " C ");
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), ellipse->cx + ellipse->rx));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cx + rx));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), ellipse->cy - RSVG_ARC_MAGIC * ellipse->ry));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cy - RSVG_ARC_MAGIC * ry));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), ellipse->cx + RSVG_ARC_MAGIC * ellipse->rx));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cx + RSVG_ARC_MAGIC * rx));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), ellipse->cy - ellipse->ry));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cy - ry));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), ellipse->cx));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cx));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), ellipse->cy - ellipse->ry));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cy - ry));
 
 	g_string_append (d, " C ");
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), ellipse->cx - RSVG_ARC_MAGIC * ellipse->rx));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cx - RSVG_ARC_MAGIC * rx));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), ellipse->cy - ellipse->ry));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cy - ry));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), ellipse->cx - ellipse->rx));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cx - rx));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), ellipse->cy - RSVG_ARC_MAGIC * ellipse->ry));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cy - RSVG_ARC_MAGIC * ry));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), ellipse->cx - ellipse->rx));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cx - rx));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), ellipse->cy));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cy));
 
 	g_string_append (d, " C ");
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), ellipse->cx - ellipse->rx));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cx - rx));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), ellipse->cy + RSVG_ARC_MAGIC * ellipse->ry));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cy + RSVG_ARC_MAGIC * ry));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), ellipse->cx - RSVG_ARC_MAGIC * ellipse->rx));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cx - RSVG_ARC_MAGIC * rx));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), ellipse->cy + ellipse->ry));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cy + ry));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), ellipse->cx));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cx));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), ellipse->cy + ellipse->ry));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cy + ry));
 
 	g_string_append (d, " C ");
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), ellipse->cx + RSVG_ARC_MAGIC * ellipse->rx));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cx + RSVG_ARC_MAGIC * rx));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), ellipse->cy + ellipse->ry));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cy + ry));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), ellipse->cx + ellipse->rx));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cx + rx));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), ellipse->cy + RSVG_ARC_MAGIC * ellipse->ry));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cy + RSVG_ARC_MAGIC * ry));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), ellipse->cx + ellipse->rx));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cx + rx));
 	g_string_append_c (d, ' ');
-	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), ellipse->cy));
+	g_string_append (d, g_ascii_dtostr (buf, sizeof (buf), cy));
 
 	g_string_append (d, " Z");
 	
@@ -708,9 +722,8 @@ rsvg_new_ellipse (void)
 	RsvgNodeEllipse *ellipse;
 	ellipse = g_new (RsvgNodeEllipse, 1);
 	_rsvg_node_init(&ellipse->super);
-	ellipse->super.type = RSVG_NODE_PATH;
 	ellipse->super.draw = _rsvg_node_ellipse_draw;
 	ellipse->super.set_atts = _rsvg_node_ellipse_set_atts;
-	ellipse->cx = ellipse->cy = ellipse->rx = ellipse->ry = 0;
+	ellipse->cx = ellipse->cy = ellipse->rx = ellipse->ry = _rsvg_css_parse_length ("0");
 	return &ellipse->super;
 }
